/*
---------------------------------------------------------------------------
 $Id: tsp_cap.c,v 1.23 2006/09/22 23:06:46 cnepveu Exp $
---------------------------------------------------------------------------
Copyright (c) 2001-2006 Hexago Inc. All rights reserved.

  LICENSE NOTICE: You may use and modify this source code only if you
  have executed a valid license agreement with Hexago Inc. granting
  you the right to do so, the said license agreement governing such
  use and modifications.   Copyright or other intellectual property
  notices are not to be removed from the source code.
---------------------------------------------------------------------------
*/

#include <stdio.h>
#include <stdlib.h>
#include <stdarg.h>
#include <string.h>

#define _USES_SYS_SOCKET_H_

#include "platform.h"

#include "tsp_cap.h"
#include "tsp_client.h"	// tspGetStatusCode()
#include "net.h"
#include "log.h"
#include "hex_strings.h"
#include "version.h"
#include "tsp_redirect.h"

/* static functions */

/* Convert a CAPABILITY string in corresponding bit in capability flag */

static
tCapability
tspExtractCapability(char *String) {
	tCapability flags;
	char *s, *e, Token[strlen(String)+1], Value[strlen(String)+1];
	int len;

	flags = 0;
	*Token = *Value = 0;

	for(s=e=String+11; *e; e++) {
		if(*e== ' ' || *e == '\r' || *e == '\n' || *e == 0) {
			if(s!=e) {
				if(*Token && (*Value == 0)) {
					len = (int)((char *)e-(char *)s);
					memcpy(Value, s, len);
					Value[len] = 0;
				}
				if(*Token && *Value) {
					flags |= tspSetCapability(Token,Value);
					*Value = *Token = 0;
				}
			}
			s = ++e;
		}

		if((*e=='=' || *e== ' ' || *e == '\r' || *e == '\n' || *e == 0) && (e != s)) {
			len = (int)((char *)e-(char *)s);
			memcpy(Token, s, len);
			Token[len] = 0;
			s = ++e;
		}
	}
	return flags;
}

/* exports */

/* Return the capability flag corrsponding to the token */
tCapability
tspSetCapability(char *Token, char *Value) {
	if(strcmp("TUNNEL", Token)==0) {
		if(strcmp("V6V4", Value)==0)
			return TUNNEL_V6V4;
		if(strcmp("V6UDPV4", Value)==0)
			return TUNNEL_V6UDPV4;
#ifdef V4V6_SUPPORT		
		if(strcmp("V4V6", Value)==0)
			return TUNNEL_V4V6;
#endif /* V4V6_SUPPORT */
	}
	if(strcmp("AUTH", Token)==0) {
		if(strcasecmp("PASSDSS-3DES-1", Value)==0)
			return AUTH_PASSDSS_3DES_1;
		if(strcasecmp("DIGEST-MD5", Value)==0)
			return AUTH_DIGEST_MD5;
		if(strcasecmp("ANONYMOUS", Value)==0)
			return AUTH_ANONYMOUS;
		if(strcasecmp("PLAIN", Value)==0)
			return AUTH_PLAIN;
	}
	return 0;
}

tErrors
tspGetCapabilities(SOCKET socket, net_tools_t *nt, tCapability *capability, int version_index, tConf *conf, tBrokerList **broker_list) {
	char dataout[256];
	char datain[REDIRECT_RECEIVE_BUFFER_SIZE];

	snprintf(dataout, sizeof(dataout), "VERSION=%s\r\n", TspProtocolVersionStrings[version_index]);
	memset(datain, -0, sizeof(datain));

	if (nt->netsendrecv(socket, dataout, strlen(dataout), datain, sizeof(datain)) == -1) 
		return SOCKET_ERROR;

	if(memcmp("CAPABILITY ", datain, 11) == 0)
		*capability = tspExtractCapability(datain);
	else {
		if ( tspGetStatusCode(datain) == 302 )
			return TSP_VERSION_ERROR;
	
		if (tspIsRedirectStatus(tspGetStatusCode(datain))) {
			if (tspHandleRedirect(datain, conf, broker_list) == TSP_REDIRECT_OK) {
				return BROKER_REDIRECTION;
			}
			else {
				return BROKER_REDIRECTION_ERROR;
			}
		}

		Display(LOG_LEVEL_3, ELInfo, "Capability", HEX_STR_ERR_FROM_SERVER, datain);
		return TSP_ERROR;
	}

	return NO_ERROR;
}

